@@ -1,5 +1,7 @@
 /*
 Copyright (c) 2008-2011 BBR Inc.  All rights reserved.
+Copyright (c) 2012-2019 by Till Kamppeter
+Copyright (c) 2019 by Tanmay Anand.
 
 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
@@ -37,27 +39,22 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_CPP_POPPLER_VERSION_H
-#include "cpp/poppler-version.h"
+#include <poppler/cpp/poppler-version.h>
 #endif
-#include "goo/GooString.h"
-#include "goo/gmem.h"
-#include "Object.h"
-#include "Stream.h"
-#include "PDFDoc.h"
-#include "SplashOutputDev.h"
-#include "GfxState.h"
 #include <cups/ppd.h>
 #include <stdarg.h>
-#include "PDFError.h"
-#include "GlobalParams.h"
 #include <cups/raster.h>
 #include <cupsfilters/image.h>
 #include <cupsfilters/raster.h>
 #include <cupsfilters/colormanager.h>
-#include <splash/SplashTypes.h>
-#include <splash/SplashBitmap.h>
 #include <strings.h>
 #include <math.h>
+#include <poppler/cpp/poppler-document.h>
+#include <poppler/cpp/poppler-page.h>
+#include <poppler/cpp/poppler-global.h>
+#include <poppler/cpp/poppler-image.h>
+#include <poppler/cpp/poppler-page-renderer.h>
+#include <poppler/cpp/poppler-rectangle.h>
 #ifdef USE_LCMS1
 #include <lcms.h>
 #define cmsColorSpaceSignature icColorSpaceSignature
@@ -166,7 +163,7 @@ namespace {
     {15,143,47,175,7,135,39,167,13,141,45,173,5,133,37,165},
     {207,79,239,111,199,71,231,103,205,77,237,109,197,69,229,101},
     {63,191,31,159,55,183,23,151,61,189,29,157,53,181,21,149},
-    {255,127,223,95,247,119,215,87,253,125,221,93,245,117,213,85} 
+    {255,127,223,95,247,119,215,87,253,125,221,93,245,117,213,85}
   };
   unsigned int dither2[8][8] = {
     {0,32,8,40,2,34,10,42},
@@ -176,13 +173,13 @@ namespace {
     {3,35,11,43,1,33,9,41},
     {51,19,59,27,49,17,57,25},
     {15,47,7,39,13,45,5,37},
-    {63,31,55,23,61,29,53,21} 
+    {63,31,55,23,61,29,53,21}
   };
   unsigned int dither4[4][4] = {
     {0,8,2,10},
     {12,4,14,6},
     {3,11,1,9},
-    {15,7,13,5} 
+    {15,7,13,5}
   };
 
   /* for color profiles */
@@ -224,7 +221,7 @@ cmsCIExyYTRIPLE adobergb_matrix()
     cmsCIExyYTRIPLE m;
 
     double * matrix = cmMatrixAdobeRgb();
-    
+
     m.Red.x = matrix[0];
     m.Red.y = matrix[1];
     m.Red.Y = matrix[2];
@@ -251,7 +248,7 @@ cmsHPROFILE adobergb_profile()
 #else
     cmsToneCurve * Gamma = cmsBuildGamma(NULL, 2.2);
     cmsToneCurve * Gamma3[3];
-#endif    
+#endif
     Gamma3[0] = Gamma3[1] = Gamma3[2] = Gamma;
 
     // Build AdobeRGB profile
@@ -272,7 +269,7 @@ cmsHPROFILE sgray_profile()
     cmsToneCurve Gamma = cmsBuildGamma(256, 2.2);
 #else
     cmsToneCurve * Gamma = cmsBuildGamma(NULL, 2.2);
-#endif 
+#endif
     // Build sGray profile
     wp = sgray_wp();
     sgray = cmsCreateGrayProfile(&wp, Gamma);
@@ -280,30 +277,6 @@ cmsHPROFILE sgray_profile()
     return sgray;
 }
 
-#if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 23
-void CDECL myErrorFun(void *data, ErrorCategory category,
-#if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 70
-    Goffset pos, const char *msg)
-#else
-    Goffset pos, char *msg)
-#endif /* MAJOR > 0 || MINOR >= 70 */
-#else
-void CDECL myErrorFun(void *data, ErrorCategory category,
-    int pos, char *msg)
-#endif
-{
-  if (pos >= 0) {
-#if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 23
-    fprintf(stderr, "ERROR (%lld): ", pos);
-#else
-    fprintf(stderr, "ERROR (%d): ", pos);
-#endif
-  } else {
-    fprintf(stderr, "ERROR: ");
-  }
-  fprintf(stderr, "%s\n",msg);
-  fflush(stderr);
-}
 
 #ifdef USE_LCMS1
 static int lcmsErrorHandler(int ErrorCode, const char *ErrorText)
@@ -320,93 +293,6 @@ static void lcmsErrorHandler(cmsContext contextId, cmsUInt32Number ErrorCode,
 #endif
 
 
-#if 0
-static bool getColorProfilePath(ppd_file_t *ppd, GooString *path)
-{
-    // get color profile path
-    const char *colorModel;
-    const char *cupsICCQualifier2;
-    const char *cupsICCQualifier2Choice;
-    const char *cupsICCQualifier3;
-    const char *cupsICCQualifier3Choice;
-    ppd_attr_t *attr;
-    ppd_choice_t *choice;
-
-    if ((choice = ppdFindMarkedChoice(ppd,"ColorModel")) != NULL) {
-	colorModel = choice->choice;
-    } else {
-	colorModel = NULL;
-    }
-    if ((attr = ppdFindAttr(ppd,"cupsICCQualifier2",NULL)) != NULL) {
-	cupsICCQualifier2 = attr->value;
-    } else {
-	cupsICCQualifier2 = "MediaType";
-    }
-    if ((choice = ppdFindMarkedChoice(ppd,cupsICCQualifier2)) != NULL) {
-	cupsICCQualifier2Choice = choice->choice;
-    } else {
-	cupsICCQualifier2Choice = NULL;
-    }
-    if ((attr = ppdFindAttr(ppd,"cupsICCQualifier3",NULL)) != NULL) {
-	cupsICCQualifier3 = attr->value;
-    } else {
-	cupsICCQualifier3 = "Resolution";
-    }
-    if ((choice = ppdFindMarkedChoice(ppd,cupsICCQualifier3)) != NULL) {
-	cupsICCQualifier3Choice = choice->choice;
-    } else {
-	cupsICCQualifier3Choice = NULL;
-    }
-
-    for (attr = ppdFindAttr(ppd,"cupsICCProfile",NULL);attr != NULL;
-       attr = ppdFindNextAttr(ppd,"cupsICCProfile",NULL)) {
-	// check color model
-	char buf[PPD_MAX_NAME];
-	char *p, *r;
-	char *datadir;
-
-	strncpy(buf,attr->spec,sizeof(buf));
-	if ((p = strchr(buf,'.')) != NULL) {
-	    *p = '\0';
-	}
-	if (colorModel != NULL && buf[0] != '\0'
-	    && strcasecmp(buf,colorModel) != 0) continue;
-	if (p == NULL) {
-	    break;
-	} else {
-	    p++;
-	    if ((r = strchr(p,'.')) != 0) {
-		*r = '\0';
-	    }
-	}
-	if (cupsICCQualifier2Choice != NULL && p[0] != '\0'
-	    && strcasecmp(p,cupsICCQualifier2Choice) != 0) continue;
-	if (r == NULL) {
-	    break;
-	} else {
-	    r++;
-	    if ((p = strchr(r,'.')) != 0) {
-		*p = '\0';
-	    }
-	}
-	if (cupsICCQualifier3Choice == NULL || r[0] == '\0'
-	    || strcasecmp(r,cupsICCQualifier3Choice) == 0) break;
-    }
-    if (attr != NULL) {
-	// matched
-	path->clear();
-	if (attr->value[0] != '/') {
-	    if ((datadir = getenv("CUPS_DATADIR")) == NULL)
-		datadir = CUPS_DATADIR;
-	    path->append(datadir);
-	    path->append("/profiles/");
-	}
-	path->append(attr->value);
-	return true;
-    }
-    return false;
-}
-#endif
 
 static void  handleRqeuiresPageRegion() {
   ppd_choice_t *mf;
@@ -448,14 +334,13 @@ static void parseOpts(int argc, char **argv)
 {
   int num_options = 0;
   cups_option_t *options = 0;
-  GooString profilePath;
   char *profile = 0;
   const char *t = NULL;
   ppd_attr_t *attr;
 
   if (argc < 6 || argc > 7) {
-    pdfError(-1,const_cast<char *>("%s job-id user title copies options [file]"),
-      argv[0]);
+    fprintf(stderr, "ERROR: Usage: %s job-id user title copies options [file]\n",
+	    argv[0]);
     exit(1);
   }
 
@@ -464,7 +349,7 @@ static void parseOpts(int argc, char **argv)
   if (t && strcasestr(t, "pwg"))
     pwgraster = 1;
 #endif /* HAVE_CUPS_1_7 */
-    
+
   ppd = ppdOpenFile(getenv("PPD"));
   if (ppd == NULL)
     fprintf(stderr, "DEBUG: PPD file is not specified.\n");
@@ -542,10 +427,10 @@ static void parseOpts(int argc, char **argv)
 
     if (cm_calibrate == CM_CALIBRATION_ENABLED)
       cm_disabled = 1;
-    else 
+    else
       cm_disabled = cmIsPrinterCmDisabled(getenv("PRINTER"));
-       
-    if (!cm_disabled) 
+
+    if (!cm_disabled)
       cmGetPrinterIccProfile(getenv("PRINTER"), &profile, ppd);
 
     if (profile != NULL) {
@@ -573,7 +458,7 @@ static void parseOpts(int argc, char **argv)
       if (strcasestr(t, "pwg"))
 	pwgraster = 1;
       else
-	pwgraster = 0; 
+	pwgraster = 0;
     }
     cupsRasterParseIPPOptions(&header,num_options,options,pwgraster,1);
 #else
@@ -1399,7 +1284,7 @@ static unsigned int getCMSColorSpaceType(cmsColorSpaceSignature cs)
 /* select convertLine function */
 static void selectConvertFunc(cups_raster_t *raster)
 {
-  if ((colorProfile == NULL || popplerColorProfile == colorProfile) 
+  if ((colorProfile == NULL || popplerColorProfile == colorProfile)
       && (header.cupsColorOrder == CUPS_ORDER_CHUNKED
        || header.cupsNumColors == 1)) {
     if (selectSpecialCase()) return;
@@ -1477,7 +1362,7 @@ static void selectConvertFunc(cups_raster_t *raster)
             COLORSPACE_SH(dcst) |
             CHANNELS_SH(header.cupsNumColors) | BYTES_SH(bytes),
             renderingIntent,0)) == 0) {
-      pdfError(-1,const_cast<char *>("Can't create color transform"));
+      fprintf(stderr, "ERROR: Can't create color transform");
       exit(1);
     }
   } else {
@@ -1548,7 +1433,7 @@ static void selectConvertFunc(cups_raster_t *raster)
       convertCSpace = W8toK8;
       break;
     default:
-      pdfError(-1,const_cast<char *>("Specified ColorSpace is not supported"));
+      fprintf(stderr, "ERROR: Specified ColorSpace is not supported\n" );
       exit(1);
       break;
     }
@@ -1623,13 +1508,103 @@ static void selectConvertFunc(cups_raster_t *raster)
   }
 }
 
-static void writePageImage(cups_raster_t *raster, SplashBitmap *bitmap,
+static unsigned char *onebitpixel(unsigned char *src, unsigned char *dst, unsigned int width, unsigned int height){
+  unsigned char *temp;
+  temp=dst;
+  int cnt=0;
+  for(unsigned int i=0;i<height;i++){
+    for(unsigned int j=0;j<width;j+=8){
+      unsigned char tem=0;
+      for(int k=0;k<8;k++){
+        cnt++;
+          tem <<=1;
+          unsigned int var=*src;
+          if(var > dither1[i & 0xf][(j+k) & 0xf]){
+            tem |= 0x1;
+          }
+          src +=1;
+      }
+      *dst=tem;
+      dst+=1;
+    }
+  }
+  return temp;
+}
+
+
+static unsigned char *removeAlpha(unsigned char *src, unsigned char *dst, unsigned int width, unsigned int height){
+  unsigned char *temp;
+  temp=dst;
+  for(unsigned int i=0;i<height;i++){
+    for(unsigned int j=0;j<width;j++){
+      dst[0]=src[2];
+      dst[1]=src[1];
+      dst[2]=src[0];
+      src+=4;
+      dst+=3;
+    }
+  }
+  return temp;
+}
+
+static void writePageImage(cups_raster_t *raster, poppler::document *doc1,
   int pageNo)
 {
   ConvertLineFunc convertLine;
   unsigned char *lineBuf = NULL;
   unsigned char *dp;
-  unsigned int rowsize = bitmap->getRowSize();
+  unsigned int rowsize;
+
+  poppler::page *current_page =doc1->create_page(pageNo-1);
+  poppler::page_renderer pr;
+  pr.set_render_hint(poppler::page_renderer::text_antialiasing);
+
+  unsigned char *colordata,*newdata,*graydata,*onebitdata;
+  unsigned int pixel_count;
+  poppler::image im;
+  //render the page according to the colourspace and generate the requried data
+  switch (header.cupsColorSpace) {
+   case CUPS_CSPACE_K://black
+   case CUPS_CSPACE_SW://sgray
+    if(header.cupsBitsPerColor==1){ //special case for 1-bit colorspaces
+    im = pr.render_page(current_page,header.HWResolution[0],header.HWResolution[1],0,0,bytesPerLine*8,header.cupsHeight);
+    newdata = (unsigned char *)malloc(sizeof(char)*3*im.width()*im.height());
+    newdata = removeAlpha((unsigned char *)im.const_data(),newdata,im.width(),im.height());
+    graydata=(unsigned char *)malloc(sizeof(char)*im.width()*im.height());
+    cupsImageRGBToWhite(newdata,graydata,im.width()*im.height());
+    onebitdata=(unsigned char *)malloc(sizeof(char)*bytesPerLine*im.height());
+    onebitpixel(graydata,onebitdata,im.width(),im.height());
+    colordata=onebitdata;
+    rowsize=bytesPerLine;
+    }
+    else{
+      im = pr.render_page(current_page,header.HWResolution[0],header.HWResolution[1],0,0,header.cupsWidth,header.cupsHeight);
+      newdata = (unsigned char *)malloc(sizeof(char)*3*im.width()*im.height());
+      newdata = removeAlpha((unsigned char *)im.const_data(),newdata,im.width(),im.height());
+      pixel_count=im.width()*im.height();
+      graydata=(unsigned char *)malloc(sizeof(char)*im.width()*im.height());
+      cupsImageRGBToWhite(newdata,graydata,pixel_count);
+      colordata=graydata;
+      rowsize=header.cupsWidth;
+    }
+
+    break;
+   case CUPS_CSPACE_RGB:
+   case CUPS_CSPACE_ADOBERGB:
+   case CUPS_CSPACE_CMYK:
+   case CUPS_CSPACE_SRGB:
+   case CUPS_CSPACE_CMY:
+   case CUPS_CSPACE_RGBW:
+   default:
+   im = pr.render_page(current_page,header.HWResolution[0],header.HWResolution[1],0,0,header.cupsWidth,header.cupsHeight);
+   newdata = (unsigned char *)malloc(sizeof(char)*3*im.width()*im.height());
+   newdata = removeAlpha((unsigned char *)im.const_data(),newdata,im.width(),im.height());
+   pixel_count=im.width()*im.height();
+   rowsize=header.cupsWidth*3;
+   colordata=newdata;
+     break;
+  }
+
 
   if (allocLineBuf) lineBuf = new unsigned char [bytesPerLine];
   if ((pageNo & 1) == 0) {
@@ -1639,7 +1614,7 @@ static void writePageImage(cups_raster_t *raster, SplashBitmap *bitmap,
   }
   if (header.Duplex && (pageNo & 1) == 0 && swap_image_y) {
     for (unsigned int plane = 0;plane < nplanes;plane++) {
-      unsigned char *bp = (unsigned char *)(bitmap->getDataPtr());
+      unsigned char *bp = colordata;
 
       bp += rowsize * (bitmapoffset[1] + header.cupsHeight - 1) +
         popplerBitsPerPixel * bitmapoffset[0] / 8;
@@ -1654,9 +1629,9 @@ static void writePageImage(cups_raster_t *raster, SplashBitmap *bitmap,
     }
   } else {
     for (unsigned int plane = 0;plane < nplanes;plane++) {
-      unsigned char *bp = (unsigned char *)(bitmap->getDataPtr());
+      unsigned char *bp = colordata;
 
-      bp += rowsize * bitmapoffset[1] + 
+      bp += rowsize * bitmapoffset[1] +
         popplerBitsPerPixel * bitmapoffset[0] / 8;
       for (unsigned int h = 0;h < header.cupsHeight;h++) {
         for (unsigned int band = 0;band < nbands;band++) {
@@ -1671,29 +1646,40 @@ static void writePageImage(cups_raster_t *raster, SplashBitmap *bitmap,
   if (allocLineBuf) delete[] lineBuf;
 }
 
-static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
-  SplashOutputDev *out, cups_raster_t *raster)
+static void outPage(poppler::document *doc1, int pageNo,
+  cups_raster_t *raster)
 {
-  SplashBitmap *bitmap;
-  Page *page = catalog->getPage(pageNo);
-  PDFRectangle mediaBox = *page->getMediaBox();
-  int rotate = page->getRotate();
+  // Page *page = catalog->getPage(pageNo);
+  // PDFRectangle mediaBox = *page->getMediaBox();
+  int rotate = 0;
   double paperdimensions[2], /* Physical size of the paper */
     margins[4];	/* Physical margins of print */
   ppd_size_t *size;		/* Page size */
   double l, swap;
   int i;
-  bool landscape = 0;
 
+  poppler::page *current_page =doc1->create_page(pageNo-1);
+  poppler::page_box_enum box = poppler::page_box_enum::media_box;
+  poppler::rectf mediaBox = current_page->page_rect(box);
+  poppler::page::orientation_enum orient = current_page->orientation();
+  switch (orient) {
+    case poppler::page::landscape: rotate=90;
+     break;
+    case poppler::page::upside_down: rotate=180;
+     break;
+    case poppler::page::seascape: rotate=270;
+     break;
+     default:rotate=0;
+  }
   fprintf(stderr, "DEBUG: mediaBox = [ %f %f %f %f ]; rotate = %d\n",
-	  mediaBox.x1, mediaBox.y1, mediaBox.x2, mediaBox.y2, rotate);
-  l = mediaBox.x2 - mediaBox.x1;
+	  mediaBox.left(), mediaBox.top(), mediaBox.right(), mediaBox.bottom(), rotate);
+  l = mediaBox.width();
   if (l < 0) l = -l;
   if (rotate == 90 || rotate == 270)
     header.PageSize[1] = (unsigned)l;
   else
     header.PageSize[0] = (unsigned)l;
-  l = mediaBox.y2 - mediaBox.y1;
+  l = mediaBox.height();
   if (l < 0) l = -l;
   if (rotate == 90 || rotate == 270)
     header.PageSize[0] = (unsigned)l;
@@ -1718,7 +1704,6 @@ static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
        * Standard size...
        */
       fprintf(stderr, "DEBUG: size = %s\n", size->name);
-      landscape = 0;
       paperdimensions[0] = size->width;
       paperdimensions[1] = size->length;
       if (pwgraster == 0) {
@@ -1746,7 +1731,6 @@ static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
 	 * Standard size in landscape orientation...
 	 */
 	fprintf(stderr, "DEBUG: landscape size = %s\n", size->name);
-	landscape = 1;
 	paperdimensions[0] = size->width;
 	paperdimensions[1] = size->length;
 	if (pwgraster == 0) {
@@ -1761,7 +1745,6 @@ static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
 	 * Custom size...
 	 */
 	fprintf(stderr, "DEBUG: size = Custom\n");
-	landscape = 0;
 	paperdimensions[1] = size->length;
 	for (i = 0; i < 2; i ++)
 	  paperdimensions[i] = header.PageSize[i];
@@ -1802,10 +1785,7 @@ static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
     }
   }
 
-  doc->displayPage(out,pageNo,header.HWResolution[0],
-		   header.HWResolution[1],(landscape == 0 ? 0 : 90),
-		   true,true,true);
-  bitmap = out->getBitmap();
+
   bitmapoffset[0] = margins[0] / 72.0 * header.HWResolution[0];
   bitmapoffset[1] = margins[3] / 72.0 * header.HWResolution[1];
 
@@ -1849,12 +1829,12 @@ static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
     header.cupsBytesPerLine *= header.cupsNumColors;
   }
   if (!cupsRasterWriteHeader2(raster,&header)) {
-      pdfError(-1,const_cast<char *>("Can't write page %d header"),pageNo);
+      fprintf(stderr, "ERROR: Can't write page %d header\n",pageNo );
       exit(1);
   }
 
   /* write page image */
-  writePageImage(raster,bitmap,pageNo);
+  writePageImage(raster,doc1,pageNo);
 }
 
 static void setPopplerColorProfile()
@@ -1936,29 +1916,19 @@ static void setPopplerColorProfile()
     popplerColorProfile = NULL;
     break;
   default:
-    pdfError(-1,const_cast<char *>("Specified ColorSpace is not supported"));
+    fprintf(stderr, "ERROR: Specified ColorSpace is not supported\n" );
     exit(1);
     break;
   }
-  if (popplerColorProfile != NULL) {
-    GfxColorSpace::setDisplayProfile(popplerColorProfile);
-  }
 }
 
 int main(int argc, char *argv[]) {
-  PDFDoc *doc;
-  SplashOutputDev *out;
-  SplashColor paperColor;
+  poppler::document *doc1;
   int i;
   int npages;
   cups_raster_t *raster;
-  enum SplashColorMode cmode;
-  int rowpad;
-  Catalog *catalog;
 
-  setErrorCallback(::myErrorFun,NULL);
   cmsSetLogErrorHandler(lcmsErrorHandler);
-  globalParams = new GlobalParams();
   parseOpts(argc, argv);
 
   if (argc == 6) {
@@ -1970,43 +1940,41 @@ int main(int argc, char *argv[]) {
 
     fd = cupsTempFd(name,sizeof(name));
     if (fd < 0) {
-      pdfError(-1,const_cast<char *>("Can't create temporary file"));
+      fprintf(stderr, "ERROR: Can't create temporary file\n");
       exit(1);
     }
 
     /* copy stdin to the tmp file */
     while ((n = read(0,buf,BUFSIZ)) > 0) {
       if (write(fd,buf,n) != n) {
-        pdfError(-1,const_cast<char *>("Can't copy stdin to temporary file"));
+        fprintf(stderr, "ERROR: Can't copy stdin to temporary file\n" );
         close(fd);
 	exit(1);
       }
     }
     close(fd);
-    doc = new PDFDoc(new GooString(name));
+    doc1=poppler::document::load_from_file(name,"","");
     /* remove name */
     unlink(name);
   } else {
-    GooString *fileName = new GooString(argv[6]);
     /* argc == 7 filenmae is specified */
     FILE *fp;
 
     if ((fp = fopen(argv[6],"rb")) == 0) {
-        pdfError(-1,const_cast<char *>("Can't open input file %s"),argv[6]);
+        fprintf(stderr, "ERROR: Can't open input file %s\n",argv[6]);
 	exit(1);
     }
     parsePDFTOPDFComment(fp);
     fclose(fp);
-    doc = new PDFDoc(fileName,NULL,NULL);
+    doc1=poppler::document::load_from_file(argv[6],"","");
   }
 
-  if (!doc->isOk()) {
+  if (doc1==NULL) {
     exitCode = 1;
     goto err1;
   }
 
-  catalog = doc->getCatalog();
-  npages = doc->getNumPages();
+  npages = doc1->pages();
 
   /* fix NumCopies, Collate ccording to PDFTOPDFComments */
   header.NumCopies = deviceCopies;
@@ -2020,7 +1988,7 @@ int main(int argc, char *argv[]) {
      && header.cupsBitsPerColor != 4
      && header.cupsBitsPerColor != 8
      && header.cupsBitsPerColor != 16) {
-    pdfError(-1,const_cast<char *>("Specified color format is not supported"));
+    fprintf(stderr, "ERROR: Specified color format is not supported\n");
     exit(1);
   }
   if (header.cupsColorOrder == CUPS_ORDER_PLANAR) {
@@ -2055,7 +2023,7 @@ int main(int argc, char *argv[]) {
     if (header.cupsColorOrder != CUPS_ORDER_CHUNKED
        || (header.cupsBitsPerColor != 8
           && header.cupsBitsPerColor != 16)) {
-      pdfError(-1,const_cast<char *>("Specified color format is not supported"));
+      fprintf(stderr, "ERROR: Specified color format is not supported\n");
       exit(1);
     }
   case CUPS_CSPACE_RGB:
@@ -2071,12 +2039,6 @@ int main(int argc, char *argv[]) {
   case CUPS_CSPACE_RGBW:
   case CUPS_CSPACE_GMCK:
   case CUPS_CSPACE_GMCS:
-    cmode = splashModeRGB8;
-    rowpad = 4;
-    /* set paper color white */
-    paperColor[0] = 255;
-    paperColor[1] = 255;
-    paperColor[2] = 255;
     popplerBitsPerPixel = 24;
     popplerNumColors = 3;
     break;
@@ -2086,21 +2048,17 @@ int main(int argc, char *argv[]) {
   case CUPS_CSPACE_WHITE:
   case CUPS_CSPACE_GOLD:
   case CUPS_CSPACE_SILVER:
-    if (header.cupsBitsPerColor == 1 
+    if (header.cupsBitsPerColor == 1
        && header.cupsBitsPerPixel == 1) {
-      cmode = splashModeMono1;
       popplerBitsPerPixel = 1;
     } else {
-      cmode = splashModeMono8;
       popplerBitsPerPixel = 8;
     }
     /* set paper color white */
-    paperColor[0] = 255;
-    rowpad = 1;
     popplerNumColors = 1;
     break;
   default:
-    pdfError(-1,const_cast<char *>("Specified ColorSpace is not supported"));
+    fprintf(stderr, "ERROR: Specified ColorSpace is not supported\n");
     exit(1);
     break;
   }
@@ -2109,28 +2067,19 @@ int main(int argc, char *argv[]) {
     setPopplerColorProfile();
   }
 
-  out = new SplashOutputDev(cmode,rowpad/* row padding */,
-    false,paperColor,true
-#if POPPLER_VERSION_MAJOR == 0 && POPPLER_VERSION_MINOR <= 30
-    ,false
-#endif
-    );
-  out->startDoc(doc);
-
   if ((raster = cupsRasterOpen(1, pwgraster ? CUPS_RASTER_WRITE_PWG :
 			       CUPS_RASTER_WRITE)) == 0) {
-        pdfError(-1,const_cast<char *>("Can't open raster stream"));
+        fprintf(stderr, "ERROR: Can't open raster stream\n");
 	exit(1);
   }
   selectConvertFunc(raster);
   for (i = 1;i <= npages;i++) {
-    outPage(doc,catalog,i,out,raster);
+    outPage(doc1,i,raster);
   }
   cupsRasterClose(raster);
 
-  delete out;
 err1:
-  delete doc;
+  delete doc1;
   if (ppd != NULL) {
     ppdClose(ppd);
   }
@@ -2144,12 +2093,6 @@ err1:
     cmsDeleteTransform(colorTransform);
   }
 
-#if POPPLER_VERSION_MAJOR == 0 && POPPLER_VERSION_MINOR < 69
-  // Check for memory leaks
-  Object::memCheck(stderr);
-  gMemReport(stderr);
-#endif
-
   return exitCode;
 }
 
@@ -2162,20 +2105,20 @@ err1:
 
 void * operator new(size_t size) _GLIBCXX_THROW (std::bad_alloc)
 {
-  return gmalloc(size);
+  return malloc(size);
 }
 
 void operator delete(void *p) throw ()
 {
-  gfree(p);
+  free(p);
 }
 
 void * operator new[](size_t size) _GLIBCXX_THROW (std::bad_alloc)
 {
-  return gmalloc(size);
+  return malloc(size);
 }
 
 void operator delete[](void *p) throw ()
 {
-  gfree(p);
+  free(p);
 }
